@@ -25,6 +25,7 @@
 	unsigned long long freeDiskSpace;
 	NSMutableArray *cachedTrackList;
 	Directory *cachedDataFileList;
+  NSMutableArray *albums;
 	// this is set when we are getting the track list so
 	// we know the cache should not be used
 	BOOL downloadingTracks;
@@ -95,6 +96,8 @@
 - (BOOL)isAudioType:(LIBMTP_filetype_t)filetype;
 - (BOOL)isMTPDevice;
 - (BOOL)isSupportedFileType:(LIBMTP_filetype_t)type;
+- (void)downloadAlbumList;
+- (void)addTrack:(Track *)track ToAlbumWithName:(NSString *)albumName;
 
 int progress (u_int64_t sent, u_int64_t total, const char* buf, unsigned len, void *data);
 int mtp_progress (uint64_t const sent, uint64_t const total, void const * const data);
