@@ -24,6 +24,8 @@
 #import "UnicodeWrapper.h"
 #import "Album.h"
 
+#include "ptp.h"
+
 // number of tracks in tracklist downloaded in between progress bar updates (MTP only)
 #define TRACKLIST_PROGRESS_UPDATE_INTERVAL	10
 
@@ -1903,6 +1905,7 @@ int mtp_progress (uint64_t const sent, uint64_t const total, void const * const
       [newAlbum addTrack:newTrack];
       [newTrack release];
     }
+    [newAlbum setState:NJB_PL_UNCHANGED];
 			
     [albums addObject:newAlbum];
     [newAlbum release];
@@ -1973,6 +1976,49 @@ int mtp_progress (uint64_t const sent, uint64_t const total, void const * const
       NSLog(@"Could not create new album");
       return;
     }
+    else
+    {
+      // let's set the artist too if we can
+      
+      uint32_t propnum = 0;
+      uint16_t *props = NULL;
+      uint32_t propcounter = 0;
+      int propfound = 0;
+      
+      uint16_t ret = ptp_mtp_getobjectpropssupported ((PTPParams *) device->params, PTP_OFC_MTP_AbstractAudioAlbum,
+                                                      &propnum, &props);
+      
+      if (ret != PTP_RC_OK || props == NULL) {
+        NSLog(@"could not get supported properties for type PTP_OFC_MTP_AbstractAudioAlbum.");
+        return;
+      }
+      
+      for (propcounter = 0; propcounter < propnum; propcounter++)
+      {
+        if (props[propcounter] == PTP_OPC_Artist)
+        {
+          propfound = 1;
+          break;
+        }
+      }
+      
+      free(props);
+      
+      if (propfound)
+      {
+        PTPPropertyValue val;
+        val.str = strdup([[track artist] UTF8String]);
+        ret = ptp_mtp_setobjectpropvalue ((PTPParams *) device->params, libmtpAlbum->album_id,
+                                          PTP_OPC_Artist, &val, PTP_DTC_STR);
+        
+        free(val.str);
+        
+        if (ret != PTP_RC_OK)
+          NSLog(@"could not set album artist, return %d", ret);
+        else
+          NSLog(@"set album artist name");
+      }
+    }
   }
   else
   {
