@@ -648,12 +648,13 @@
 -(NSMutableArray *)genreArrayFromFrame
 {  //Splits the genre string into an array of string. Number are converted to strings base on the dictionary. if the dictionary is NULL the system used the inbuilt dictionary.
     //check that user passed valid variables
-    NSString * String = [[self getTextFromFrame] retain];
+    NSString * String = [self getTextFromFrame];
+  
+    if ([String length] == 0)
+      return [NSMutableArray arrayWithCapacity:0];
        
     //obtain a cstring representation of the genre string
-    NSMutableData *Data = [NSMutableData dataWithLength:[String length]];
-    char * tempPointer = (char *) [Data bytes];
-    [String getCString:tempPointer];
+    const char * tempPointer = [String cString];
     int start = 0;
     int end = 0;
     int foundNull = 0;
@@ -706,7 +707,6 @@
         end ++;
     }
     if (end>start) [Array addObject:[NSString stringWithCString:tempPointer + start length:end - start]];
-    [String release];
     return Array;
 }
 
